/*!
 * jQuery QueryBuilder 2.5.2
 * Copyright 2014-2018 Damien "Mistic" Sorel (http://www.strangeplanet.fr)
 * Licensed under MIT (https://opensource.org/licenses/MIT)
 */
$theme-name: dark;

$group-background-color: rgba(50, 70, 80, .5);
$group-border-color: #00164A;

$rule-background-color: rgba(40, 40, 40, .9);
$rule-border-color: #111;

$error-border-color: #800;
$error-background-color: #322;

$ticks-color: #222;

@import 'default';
